@@ -159,12 +159,14 @@ impl Control for CurveEditor {
     }
 
     fn draw(&self, ctx: &mut DrawingContext) {
+        ctx.transform_stack.push(Matrix3::identity());
         self.update_matrices();
         self.draw_background(ctx);
         self.draw_grid(ctx);
         self.draw_curve(ctx);
         self.draw_keys(ctx);
         self.draw_operation(ctx);
+        ctx.transform_stack.pop();
     }
 
     fn handle_routed_message(&mut self, ui: &mut UserInterface, message: &mut UiMessage) {
@@ -1145,9 +1147,8 @@ impl CurveEditorBuilder {
             grid_brush: Brush::Solid(Color::from_rgba(110, 110, 110, 50)),
             selection: None,
             text: RefCell::new(
-                FormattedTextBuilder::new()
+                FormattedTextBuilder::new(ctx.default_font())
                     .with_brush(Brush::Solid(Color::opaque(100, 100, 100)))
-                    .with_font(crate::DEFAULT_FONT.clone())
                     .build(),
             ),
             context_menu: ContextMenu {
