@@ -9,9 +9,6 @@
 #![allow(clippy::from_over_into)]
 #![allow(clippy::new_without_default)]
 
-#[macro_use]
-extern crate lazy_static;
-
 pub use fyrox_core as core;
 
 pub mod border;
@@ -72,21 +69,20 @@ use crate::{
         UiMessage,
     },
     popup::{Placement, PopupMessage},
-    ttf::{Font, SharedFont},
+    ttf::{Font, FontBuilder, SharedFont},
     widget::{Widget, WidgetBuilder, WidgetMessage},
 };
 use copypasta::ClipboardContext;
 use fxhash::{FxHashMap, FxHashSet};
+use fyrox_core::algebra::Matrix3;
+use std::collections::hash_map::Entry;
 use std::{
     any::{Any, TypeId},
     cell::Cell,
     collections::VecDeque,
     fmt::Debug,
     ops::{Deref, DerefMut, Index, IndexMut},
-    sync::{
-        mpsc::{self, Receiver, Sender, TryRecvError},
-        Arc, Mutex,
-    },
+    sync::mpsc::{self, Receiver, Sender, TryRecvError},
 };
 
 // TODO: Make this part of UserInterface struct.
@@ -204,6 +200,54 @@ impl Thickness {
             bottom: 0.0,
         }
     }
+    pub fn top_right(v: f32) -> Self {
+        Self {
+            left: 0.0,
+            top: v,
+            right: v,
+            bottom: 0.0,
+        }
+    }
+    pub fn top_left(v: f32) -> Self {
+        Self {
+            left: v,
+            top: v,
+            right: 0.0,
+            bottom: 0.0,
+        }
+    }
+    pub fn bottom_right(v: f32) -> Self {
+        Self {
+            left: 0.0,
+            top: 0.0,
+            right: v,
+            bottom: v,
+        }
+    }
+    pub fn bottom_left(v: f32) -> Self {
+        Self {
+            left: v,
+            top: 0.0,
+            right: 0.0,
+            bottom: v,
+        }
+    }
+    pub fn top_bottom(v: f32) -> Self {
+        Self {
+            left: 0.0,
+            top: v,
+            right: 0.0,
+            bottom: v,
+        }
+    }
+    pub fn left_right(v: f32) -> Self {
+        Self {
+            left: v,
+            top: 0.0,
+            right: v,
+            bottom: 0.0,
+        }
+    }
 
     pub fn offset(&self) -> Vector2<f32> {
         Vector2::new(self.left, self.top)
@@ -374,9 +418,9 @@ impl Default for DragContext {
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct MouseState {
-    left: ButtonState,
-    right: ButtonState,
-    middle: ButtonState,
+    pub left: ButtonState,
+    pub right: ButtonState,
+    pub middle: ButtonState,
     // TODO Add rest of buttons
 }
 
@@ -418,6 +462,22 @@ impl UiNode {
     pub fn cast_mut<T: Control>(&mut self) -> Option<&mut T> {
         self.0.as_any_mut().downcast_mut::<T>()
     }
+
+    pub fn query_component<T>(&self) -> Option<&T>
+    where
+        T: 'static,
+    {
+        self.0
+            .query_component(TypeId::of::<T>())
+            .and_then(|c| c.downcast_ref::<T>())
+    }
+
+    pub fn has_component<T>(&self) -> bool
+    where
+        T: 'static,
+    {
+        self.query_component::<T>().is_some()
+    }
 }
 
 pub struct BuildContext<'a> {
@@ -425,6 +485,10 @@ pub struct BuildContext<'a> {
 }
 
 impl<'a> BuildContext<'a> {
+    pub fn default_font(&self) -> SharedFont {
+        self.ui.default_font.clone()
+    }
+
     pub fn add_node(&mut self, node: UiNode) -> Handle<UiNode> {
         self.ui.add_node(node)
     }
@@ -436,6 +500,14 @@ impl<'a> BuildContext<'a> {
     pub fn copy(&mut self, node: Handle<UiNode>) -> Handle<UiNode> {
         self.ui.copy_node(node)
     }
+
+    pub fn try_get_node(&self, node: Handle<UiNode>) -> Option<&UiNode> {
+        self.ui.try_get_node(node)
+    }
+
+    pub fn try_get_node_mut(&mut self, node: Handle<UiNode>) -> Option<&mut UiNode> {
+        self.ui.nodes.try_borrow_mut(node)
+    }
 }
 
 impl<'a> Index<Handle<UiNode>> for BuildContext<'a> {
@@ -502,6 +574,12 @@ enum LayoutEvent {
     VisibilityChanged(Handle<UiNode>),
 }
 
+#[derive(Clone, Debug)]
+struct DoubleClickEntry {
+    timer: f32,
+    click_count: u32,
+}
+
 pub struct UserInterface {
     screen_size: Vector2<f32>,
     nodes: Pool<UiNode>,
@@ -528,14 +606,26 @@ pub struct UserInterface {
     layout_events_receiver: Receiver<LayoutEvent>,
     layout_events_sender: Sender<LayoutEvent>,
     need_update_global_transform: bool,
+    pub default_font: SharedFont,
+    double_click_entries: FxHashMap<MouseButton, DoubleClickEntry>,
+    pub double_click_time_slice: f32,
 }
 
-lazy_static! {
-    pub static ref DEFAULT_FONT: SharedFont = {
-        let font_bytes = std::include_bytes!("./built_in_font.ttf").to_vec();
-        let font = Font::from_memory(font_bytes, 16.0, Font::default_char_set()).unwrap();
-        Arc::new(Mutex::new(font)).into()
-    };
+fn is_on_screen(node: &UiNode, nodes: &Pool<UiNode>) -> bool {
+    // Crawl up on tree and check if current bounds are intersects with every screen bound
+    // of parents chain. This is needed because some control can move their children outside of
+    // their bounds (like scroll viewer, etc.) and single intersection test of parent bounds with
+    // current bounds is not enough.
+    let bounds = node.clip_bounds();
+    let mut parent = node.parent();
+    while parent.is_some() {
+        let parent_node = nodes.borrow(parent);
+        if !parent_node.clip_bounds().intersects(bounds) {
+            return false;
+        }
+        parent = parent_node.parent();
+    }
+    true
 }
 
 fn draw_node(
@@ -550,18 +640,8 @@ fn draw_node(
         return;
     }
 
-    // Crawl up on tree and check if current bounds are intersects with every screen bound
-    // of parents chain. This is needed because some control can move their children outside of
-    // their bounds (like scroll viewer, etc.) and single intersection test of parent bounds with
-    // current bounds is not enough.
-    let bounds = node.screen_bounds();
-    let mut parent = node.parent();
-    while parent.is_some() {
-        let parent_node = nodes.borrow(parent);
-        if !parent_node.screen_bounds().intersects(bounds) {
-            return;
-        }
-        parent = parent_node.parent();
+    if !is_on_screen(node, nodes) {
+        return;
     }
 
     let start_index = drawing_context.get_commands().len();
@@ -576,6 +656,8 @@ fn draw_node(
         false
     };
 
+    drawing_context.transform_stack.push(node.visual_transform);
+
     node.draw(drawing_context);
 
     let end_index = drawing_context.get_commands().len();
@@ -591,6 +673,8 @@ fn draw_node(
         }
     }
 
+    drawing_context.transform_stack.pop();
+
     if pushed {
         drawing_context.pop_opacity();
     }
@@ -615,6 +699,7 @@ impl UserInterface {
     pub fn new(screen_size: Vector2<f32>) -> UserInterface {
         let (sender, receiver) = mpsc::channel();
         let (layout_events_sender, layout_events_receiver) = mpsc::channel();
+        let default_font = SharedFont::new(FontBuilder::new().build_builtin().unwrap());
         let mut ui = UserInterface {
             screen_size,
             sender,
@@ -641,6 +726,9 @@ impl UserInterface {
             layout_events_receiver,
             layout_events_sender,
             need_update_global_transform: Default::default(),
+            default_font,
+            double_click_entries: Default::default(),
+            double_click_time_slice: 0.75,
         };
         ui.root_canvas = ui.add_node(UiNode::new(Canvas::new(WidgetBuilder::new().build())));
         ui
@@ -716,7 +804,7 @@ impl UserInterface {
         }
     }
 
-    fn update_transform(&mut self) {
+    fn update_visual_transform(&mut self) {
         scope_profile!();
 
         self.stack.clear();
@@ -731,13 +819,18 @@ impl UserInterface {
             if widget.is_globally_visible() {
                 self.stack.extend_from_slice(widget.children());
 
-                let screen_position = if let Some(parent) = parent {
-                    widget.actual_local_position() + parent.screen_position()
+                let mut layout_transform = widget.layout_transform;
+
+                layout_transform[6] = widget.actual_local_position().x;
+                layout_transform[7] = widget.actual_local_position().y;
+
+                let visual_transform = if let Some(parent) = parent {
+                    parent.visual_transform * widget.render_transform * layout_transform
                 } else {
-                    widget.actual_local_position()
+                    widget.render_transform * layout_transform
                 };
 
-                widget.screen_position = screen_position;
+                widget.visual_transform = visual_transform;
             }
         }
     }
@@ -785,19 +878,30 @@ impl UserInterface {
 
         self.screen_size = screen_size;
 
+        for entry in self.double_click_entries.values_mut() {
+            entry.timer -= dt;
+        }
+
         self.handle_layout_events();
 
         self.measure_node(self.root_canvas, screen_size);
-        self.arrange_node(
+        let arrangement_changed = self.arrange_node(
             self.root_canvas,
             &Rect::new(0.0, 0.0, screen_size.x, screen_size.y),
         );
 
         if self.need_update_global_transform {
-            self.update_transform();
+            self.update_visual_transform();
             self.need_update_global_transform = false;
         }
 
+        if arrangement_changed {
+            self.calculate_clip_bounds(
+                self.root_canvas,
+                Rect::new(0.0, 0.0, self.screen_size.x, self.screen_size.y),
+            );
+        }
+
         let sender = self.sender.clone();
         for node in self.nodes.iter_mut() {
             node.update(dt, &sender)
@@ -828,14 +932,10 @@ impl UserInterface {
     pub fn draw(&mut self) -> &DrawingContext {
         scope_profile!();
 
-        self.calculate_clip_bounds(
-            self.root_canvas,
-            Rect::new(0.0, 0.0, self.screen_size.x, self.screen_size.y),
-        );
         self.drawing_context.clear();
 
         for node in self.nodes.iter_mut() {
-            node.command_indices.borrow_mut().clear();
+            node.command_indices.get_mut().clear();
         }
 
         // Draw everything except top-most nodes.
@@ -847,6 +947,11 @@ impl UserInterface {
         self.stack.push(self.root());
         while let Some(node_handle) = self.stack.pop() {
             let node = &self.nodes[node_handle];
+
+            if !is_on_screen(node, &self.nodes) {
+                continue;
+            }
+
             if node.is_draw_on_top() {
                 draw_node(&self.nodes, node_handle, &mut self.drawing_context);
             }
@@ -926,12 +1031,20 @@ impl UserInterface {
                 size.y = node.height();
             }
 
+            size = transform_size(size, &node.layout_transform);
+
             size = node.arrange_override(self, size);
 
             size.x = size.x.min(final_rect.w());
             size.y = size.y.min(final_rect.h());
 
-            let mut origin = final_rect.position + node.margin().offset();
+            let transformed_rect =
+                Rect::new(0.0, 0.0, size.x, size.y).transform(&node.layout_transform);
+
+            size = transformed_rect.size;
+
+            let mut origin =
+                final_rect.position - transformed_rect.position + node.margin().offset();
 
             match node.horizontal_alignment() {
                 HorizontalAlignment::Center | HorizontalAlignment::Stretch => {
@@ -985,11 +1098,17 @@ impl UserInterface {
                 },
             );
 
+            size = transform_size(size, &node.layout_transform);
+
             size.x = clampf(size.x, node.min_size().x, node.max_size().x);
             size.y = clampf(size.y, node.min_size().y, node.max_size().y);
 
             let mut desired_size = node.measure_override(self, size);
 
+            desired_size = Rect::new(0.0, 0.0, desired_size.x, desired_size.y)
+                .transform(&node.layout_transform)
+                .size;
+
             if !node.width().is_nan() {
                 desired_size.x = node.width();
             }
@@ -1021,7 +1140,7 @@ impl UserInterface {
         let widget = self.nodes.borrow(node_handle);
 
         if widget.is_globally_visible() {
-            clipped = !widget.screen_bounds().contains(pt);
+            clipped = !widget.clip_bounds().contains(pt);
 
             if !clipped {
                 for command_index in widget.command_indices.borrow().iter() {
@@ -1079,7 +1198,7 @@ impl UserInterface {
 
         if !widget.is_hit_test_visible()
             || !widget.enabled()
-            || !widget.screen_bounds().intersects(Rect {
+            || !widget.clip_bounds().intersects(Rect {
                 position: Default::default(),
                 size: self.screen_size,
             })
@@ -1110,15 +1229,19 @@ impl UserInterface {
         self.cursor_position
     }
 
+    pub fn hit_test_unrestricted(&self, pt: Vector2<f32>) -> Handle<UiNode> {
+        // We're not restricted to any node, just start from root.
+        let mut level = 0;
+        self.pick_node(self.root_canvas, pt, &mut level)
+    }
+
     pub fn hit_test(&self, pt: Vector2<f32>) -> Handle<UiNode> {
         scope_profile!();
 
         if self.nodes.is_valid_handle(self.captured_node) {
             self.captured_node
         } else if self.picking_stack.is_empty() {
-            // We're not restricted to any node, just start from root.
-            let mut level = 0;
-            self.pick_node(self.root_canvas, pt, &mut level)
+            self.hit_test_unrestricted(pt)
         } else {
             // We have some picking restriction chain.
             // Go over picking stack and try each entry. This will help with picking
@@ -1205,8 +1328,15 @@ impl UserInterface {
     /// Recursively calculates clipping bounds for every node.
     fn calculate_clip_bounds(&self, node: Handle<UiNode>, parent_bounds: Rect<f32>) {
         let node = &self.nodes[node];
-        node.clip_bounds
-            .set(node.screen_bounds().clip_by(parent_bounds));
+
+        let screen_bounds = if node.clip_to_bounds {
+            node.screen_bounds()
+        } else {
+            Rect::new(0.0, 0.0, self.screen_size.x, self.screen_size.y)
+        };
+
+        node.clip_bounds.set(screen_bounds.clip_by(parent_bounds));
+
         for &child in node.children() {
             self.calculate_clip_bounds(child, node.clip_bounds.get());
         }
@@ -1325,6 +1455,15 @@ impl UserInterface {
         }
     }
 
+    fn make_lowermost(&mut self, node: Handle<UiNode>) {
+        let parent = self.node(node).parent();
+        if parent.is_some() {
+            let parent = &mut self.nodes[parent];
+            parent.remove_child(node);
+            parent.add_child(node, true);
+        }
+    }
+
     fn bubble_message(&mut self, message: &mut UiMessage) {
         scope_profile!();
 
@@ -1397,17 +1536,22 @@ impl UserInterface {
                                 }
                             }
                         }
-                        WidgetMessage::TopMost => {
+                        WidgetMessage::Topmost => {
                             if message.destination().is_some() {
                                 self.make_topmost(message.destination());
                             }
                         }
+                        WidgetMessage::Lowermost => {
+                            if message.destination().is_some() {
+                                self.make_lowermost(message.destination());
+                            }
+                        }
                         WidgetMessage::Unlink => {
                             if message.destination().is_some() {
                                 self.unlink_node(message.destination());
 
                                 let node = &self.nodes[message.destination()];
-                                let new_position = node.screen_position;
+                                let new_position = node.screen_position();
                                 self.send_message(WidgetMessage::desired_position(
                                     message.destination(),
                                     MessageDirection::ToWidget,
@@ -1430,6 +1574,22 @@ impl UserInterface {
                                 self.remove_node(message.destination());
                             }
                         }
+                        WidgetMessage::ContextMenu(context_menu) => {
+                            if message.destination().is_some() {
+                                let node = self.nodes.borrow_mut(message.destination());
+
+                                let prev_context_menu = node.context_menu();
+
+                                node.set_context_menu(*context_menu);
+
+                                if prev_context_menu.is_some() {
+                                    self.send_message(WidgetMessage::remove(
+                                        prev_context_menu,
+                                        MessageDirection::ToWidget,
+                                    ));
+                                }
+                            }
+                        }
                         WidgetMessage::Center => {
                             if message.destination().is_some() {
                                 let node = self.node(message.destination());
@@ -1588,6 +1748,25 @@ impl UserInterface {
         self.captured_node
     }
 
+    // Tries to set new picked node (a node under the cursor) and returns `true` if the node was
+    // changed.
+    fn try_set_picked_node(&mut self, node: Handle<UiNode>) -> bool {
+        if self.picked_node != node {
+            self.picked_node = node;
+            self.reset_double_click_entries();
+            true
+        } else {
+            false
+        }
+    }
+
+    fn reset_double_click_entries(&mut self) {
+        for entry in self.double_click_entries.values_mut() {
+            entry.timer = self.double_click_time_slice;
+            entry.click_count = 0;
+        }
+    }
+
     /// Translates raw window event into some specific UI message. This is one of the
     /// most important methods of UI. You must call it each time you received a message
     /// from a window.
@@ -1605,7 +1784,40 @@ impl UserInterface {
 
                 match state {
                     ButtonState::Pressed => {
-                        self.picked_node = self.hit_test(self.cursor_position);
+                        let picked_changed =
+                            self.try_set_picked_node(self.hit_test(self.cursor_position));
+
+                        if !picked_changed {
+                            match self.double_click_entries.entry(button) {
+                                Entry::Occupied(e) => {
+                                    let entry = e.into_mut();
+                                    if entry.timer > 0.0 {
+                                        entry.click_count += 1;
+                                        if entry.click_count >= 2 {
+                                            entry.click_count = 0;
+                                            entry.timer = self.double_click_time_slice;
+
+                                            self.send_message(WidgetMessage::double_click(
+                                                self.picked_node,
+                                                MessageDirection::FromWidget,
+                                                button,
+                                            ));
+                                        }
+                                    } else {
+                                        entry.timer = self.double_click_time_slice;
+                                        entry.click_count = 1;
+                                    }
+                                }
+                                Entry::Vacant(entry) => {
+                                    // A button was clicked for the first time, no double click
+                                    // in this case.
+                                    entry.insert(DoubleClickEntry {
+                                        timer: self.double_click_time_slice,
+                                        click_count: 1,
+                                    });
+                                }
+                            }
+                        }
 
                         // Try to find draggable node in hierarchy starting from picked node.
                         if self.picked_node.is_some() {
@@ -1695,7 +1907,7 @@ impl UserInterface {
             }
             OsEvent::CursorMoved { position } => {
                 self.cursor_position = *position;
-                self.picked_node = self.hit_test(self.cursor_position);
+                self.try_set_picked_node(self.hit_test(self.cursor_position));
 
                 if !self.drag_context.is_dragging
                     && self.mouse_state.left == ButtonState::Pressed
@@ -1900,7 +2112,7 @@ impl UserInterface {
                 self.prev_picked_node = Handle::NONE;
             }
             if self.picked_node == handle {
-                self.picked_node = Handle::NONE;
+                self.try_set_picked_node(Handle::NONE);
             }
             if self.captured_node == handle {
                 self.captured_node = Handle::NONE;
@@ -1993,6 +2205,7 @@ impl UserInterface {
         root
     }
 
+    #[allow(clippy::unnecessary_to_owned)] // False positive
     fn copy_node_recursive(
         &mut self,
         node_handle: Handle<UiNode>,
@@ -2029,6 +2242,7 @@ impl UserInterface {
         root
     }
 
+    #[allow(clippy::unnecessary_to_owned)] // False positive
     fn copy_node_recursive_with_limit(
         &mut self,
         node_handle: Handle<UiNode>,
@@ -2065,15 +2279,212 @@ impl UserInterface {
     }
 }
 
+fn is_approx_zero(v: f32) -> bool {
+    v.abs() <= 10.0 * f32::EPSILON
+}
+
+fn are_close(value1: f32, value2: f32) -> bool {
+    //in case they are Infinities (then epsilon check does not work)
+    if value1 == value2 {
+        return true;
+    }
+    // This computes (|value1-value2| / (|value1| + |value2| + 10.0)) < DBL_EPSILON
+    let eps = (value1.abs() + value2.abs() + 10.0) * f32::EPSILON;
+    let delta = value1 - value2;
+    (-eps < delta) && (eps > delta)
+}
+
+fn greater_than_or_close(value1: f32, value2: f32) -> bool {
+    (value1 > value2) || are_close(value1, value2)
+}
+
+fn less_than_or_close(value1: f32, value2: f32) -> bool {
+    (value1 < value2) || are_close(value1, value2)
+}
+
+/// Calculates a new size for the rect after transforming it with the given matrix. Basically it
+/// finds a new rectangle that can contain the rotated rectangle.
+///
+/// # Origin
+///
+/// Original code was taken from WPF source code (FindMaximalAreaLocalSpaceRect) and ported to Rust.
+/// It handles a lot of edge cases that could occur due to the fact that the UI uses a lot of
+/// special floating-point constants like Infinity or NaN. If there would be no such values, simple
+/// `rect.transform(&matrix).size` could be used.
+fn transform_size(transform_space_bounds: Vector2<f32>, matrix: &Matrix3<f32>) -> Vector2<f32> {
+    // X (width) and Y (height) constraints for axis-aligned bounding box in dest. space
+    let mut x_constr: f32 = transform_space_bounds.x;
+    let mut y_constr: f32 = transform_space_bounds.y;
+
+    //if either of the sizes is 0, return 0,0 to avoid doing math on an empty rect (bug 963569)
+    if is_approx_zero(x_constr) || is_approx_zero(y_constr) {
+        return Vector2::new(0.0, 0.0);
+    }
+
+    let x_constr_infinite = x_constr.is_infinite();
+    let y_constr_infinite = y_constr.is_infinite();
+
+    if x_constr_infinite && y_constr_infinite {
+        return Vector2::new(f32::INFINITY, f32::INFINITY);
+    } else if x_constr_infinite
+    //assume square for one-dimensional constraint
+    {
+        x_constr = y_constr;
+    } else if y_constr_infinite {
+        y_constr = x_constr;
+    }
+
+    // We only deal with nonsingular matrices here. The nonsingular matrix is the one
+    // that has inverse (determinant != 0).
+    if !matrix.is_invertible() {
+        return Vector2::new(0.0, 0.0);
+    }
+
+    let a = matrix[(0, 0)];
+    let b = matrix[(0, 1)];
+    let c = matrix[(1, 0)];
+    let d = matrix[(1, 1)];
+
+    // Result width and height (in child/local space)
+    let mut w;
+    let mut h;
+
+    // because we are dealing with nonsingular transform matrices,
+    // we have (b==0 || c==0) XOR (a==0 || d==0)
+
+    if is_approx_zero(b) || is_approx_zero(c) {
+        // (b==0 || c==0) ==> a!=0 && d!=0
+
+        let y_cover_d = if y_constr_infinite {
+            f32::INFINITY
+        } else {
+            (y_constr / d).abs()
+        };
+        let x_cover_a = if x_constr_infinite {
+            f32::INFINITY
+        } else {
+            (x_constr / a).abs()
+        };
+
+        if is_approx_zero(b) {
+            if is_approx_zero(c) {
+                // Case: b=0, c=0, a!=0, d!=0
+
+                // No constraint relation; use maximal width and height
+
+                h = y_cover_d;
+                w = x_cover_a;
+            } else {
+                // Case: b==0, a!=0, c!=0, d!=0
+
+                // Maximizing under line (hIntercept=xConstr/c, wIntercept=xConstr/a)
+                // BUT we still have constraint: h <= yConstr/d
+
+                h = (0.5 * (x_constr / c).abs()).min(y_cover_d);
+                w = x_cover_a - ((c * h) / a);
+            }
+        } else {
+            // Case: c==0, a!=0, b!=0, d!=0
+
+            // Maximizing under line (hIntercept=yConstr/d, wIntercept=yConstr/b)
+            // BUT we still have constraint: w <= xConstr/a
+
+            w = (0.5 * (y_constr / b).abs()).min(x_cover_a);
+            h = y_cover_d - ((b * w) / d);
+        }
+    } else if is_approx_zero(a) || is_approx_zero(d) {
+        // (a==0 || d==0) ==> b!=0 && c!=0
+
+        let y_cover_b = (y_constr / b).abs();
+        let x_cover_c = (x_constr / c).abs();
+
+        if is_approx_zero(a) {
+            if is_approx_zero(d) {
+                // Case: a=0, d=0, b!=0, c!=0
+
+                // No constraint relation; use maximal width and height
+
+                h = x_cover_c;
+                w = y_cover_b;
+            } else {
+                // Case: a==0, b!=0, c!=0, d!=0
+
+                // Maximizing under line (hIntercept=yConstr/d, wIntercept=yConstr/b)
+                // BUT we still have constraint: h <= xConstr/c
+
+                h = (0.5 * (y_constr / d).abs()).min(x_cover_c);
+                w = y_cover_b - ((d * h) / b);
+            }
+        } else {
+            // Case: d==0, a!=0, b!=0, c!=0
+
+            // Maximizing under line (hIntercept=xConstr/c, wIntercept=xConstr/a)
+            // BUT we still have constraint: w <= yConstr/b
+
+            w = (0.5 * (x_constr / a).abs()).min(y_cover_b);
+            h = x_cover_c - ((a * w) / c);
+        }
+    } else {
+        let x_cover_a = (x_constr / a).abs(); // w-intercept of x-constraint line.
+        let x_cover_c = (x_constr / c).abs(); // h-intercept of x-constraint line.
+
+        let y_cover_b = (y_constr / b).abs(); // w-intercept of y-constraint line.
+        let y_cover_d = (y_constr / d).abs(); // h-intercept of y-constraint line.
+
+        // The tighest constraint governs, so we pick the lowest constraint line.
+        //
+        //   The optimal point (w,h) for which Area = w*h is maximized occurs halfway
+        //   to each intercept.
+
+        w = y_cover_b.min(x_cover_a) * 0.5;
+        h = x_cover_c.min(y_cover_d) * 0.5;
+
+        if (greater_than_or_close(x_cover_a, y_cover_b) && less_than_or_close(x_cover_c, y_cover_d))
+            || (less_than_or_close(x_cover_a, y_cover_b)
+                && greater_than_or_close(x_cover_c, y_cover_d))
+        {
+            // Constraint lines cross; since the most restrictive constraint wins,
+            // we have to maximize under two line segments, which together are discontinuous.
+            // Instead, we maximize w*h under the line segment from the two smallest endpoints.
+
+            // Since we are not (except for in corner cases) on the original constraint lines,
+            // we are not using up all the available area in transform space.  So scale our shape up
+            // until it does in at least one dimension.
+
+            let child_bounds_tr = Rect::new(0.0, 0.0, w, h).transform(matrix);
+            let expand_factor =
+                (x_constr / child_bounds_tr.size.x).min(y_constr / child_bounds_tr.size.y);
+
+            if !expand_factor.is_nan() && !expand_factor.is_infinite() {
+                w *= expand_factor;
+                h *= expand_factor;
+            }
+        }
+    }
+
+    Vector2::new(w, h)
+}
+
 #[cfg(test)]
 mod test {
     use crate::{
         border::BorderBuilder,
         core::algebra::Vector2,
         message::MessageDirection,
+        transform_size,
         widget::{WidgetBuilder, WidgetMessage},
         UserInterface,
     };
+    use fyrox_core::algebra::{Rotation2, UnitComplex};
+
+    #[test]
+    fn test_transform_size() {
+        let input = Vector2::new(100.0, 100.0);
+        let transform =
+            Rotation2::from(UnitComplex::from_angle(45.0f32.to_radians())).to_homogeneous();
+        let transformed = transform_size(input, &transform);
+        dbg!(input, transformed);
+    }
 
     #[test]
     fn center() {
