@@ -6,6 +6,7 @@ use crate::{
     HorizontalAlignment, LayoutEvent, MouseButton, MouseState, Thickness, UiNode, UserInterface,
     VerticalAlignment, BRUSH_FOREGROUND, BRUSH_PRIMARY,
 };
+use fyrox_core::algebra::{Matrix3, Point2};
 use std::{
     any::Any,
     cell::{Cell, RefCell},
@@ -112,7 +113,13 @@ pub enum WidgetMessage {
     /// level only!
     ///
     /// Direction: **From/To UI**.
-    TopMost,
+    Topmost,
+
+    /// A request to make widget lowermost. Widget can be made lowermost only in the same hierarchy
+    /// level only!
+    ///
+    /// Direction: **From/To UI**.
+    Lowermost,
 
     /// A request to detach widget from its current parent and attach to root canvas.
     ///
@@ -260,6 +267,18 @@ pub enum WidgetMessage {
     ///
     /// Direction: **From/To UI**
     Opacity(Option<f32>),
+
+    /// A request to set new layout transform.
+    LayoutTransform(Matrix3<f32>),
+
+    /// A request to set new render transform.
+    RenderTransform(Matrix3<f32>),
+
+    /// A double click of a mouse button has occurred on a widget.
+    DoubleClick { button: MouseButton },
+
+    /// A request to set new context menu for a widget. Old context menu will be removed.
+    ContextMenu(Handle<UiNode>),
 }
 
 impl WidgetMessage {
@@ -274,7 +293,8 @@ impl WidgetMessage {
     define_constructor!(WidgetMessage:Height => fn height(f32), layout: false);
     define_constructor!(WidgetMessage:DesiredPosition => fn desired_position(Vector2<f32>), layout: false);
     define_constructor!(WidgetMessage:Center => fn center(), layout: true);
-    define_constructor!(WidgetMessage:TopMost => fn topmost(), layout: false);
+    define_constructor!(WidgetMessage:Topmost => fn topmost(), layout: false);
+    define_constructor!(WidgetMessage:Lowermost => fn lowermost(), layout: false);
     define_constructor!(WidgetMessage:Enabled => fn enabled(bool), layout: false);
     define_constructor!(WidgetMessage:Name => fn name(String), layout: false);
     define_constructor!(WidgetMessage:Row => fn row(usize), layout: false);
@@ -288,6 +308,9 @@ impl WidgetMessage {
     define_constructor!(WidgetMessage:HorizontalAlignment => fn horizontal_alignment(HorizontalAlignment), layout: false);
     define_constructor!(WidgetMessage:VerticalAlignment => fn vertical_alignment(VerticalAlignment), layout: false);
     define_constructor!(WidgetMessage:Opacity => fn opacity(Option<f32>), layout: false);
+    define_constructor!(WidgetMessage:LayoutTransform => fn layout_transform(Matrix3<f32>), layout: false);
+    define_constructor!(WidgetMessage:RenderTransform => fn render_transform(Matrix3<f32>), layout: false);
+    define_constructor!(WidgetMessage:ContextMenu => fn context_menu(Handle<UiNode>), layout: false);
 
     // Internal messages. Do not use.
     define_constructor!(WidgetMessage:GotFocus => fn got_focus(), layout: false);
@@ -304,6 +327,7 @@ impl WidgetMessage {
     define_constructor!(WidgetMessage:DragStarted => fn drag_started(Handle<UiNode>), layout: false);
     define_constructor!(WidgetMessage:DragOver => fn drag_over(Handle<UiNode>), layout: false);
     define_constructor!(WidgetMessage:Drop => fn drop(Handle<UiNode>), layout: false);
+    define_constructor!(WidgetMessage:DoubleClick => fn double_click(button: MouseButton), layout: false);
 }
 
 #[derive(Debug, Clone)]
@@ -316,8 +340,6 @@ pub struct Widget {
     width: f32,
     /// Explicit height for node or automatic if NaN (means value is undefined). Default is NaN
     height: f32,
-    /// Screen position of the node
-    pub(in crate) screen_position: Vector2<f32>,
     /// Minimum width and height
     min_size: Vector2<f32>,
     /// Maximum width and height
@@ -354,6 +376,10 @@ pub struct Widget {
     tooltip: Handle<UiNode>,
     tooltip_time: f32,
     context_menu: Handle<UiNode>,
+    pub(in crate) clip_to_bounds: bool,
+    pub(in crate) layout_transform: Matrix3<f32>,
+    pub(in crate) render_transform: Matrix3<f32>,
+    pub(in crate) visual_transform: Matrix3<f32>,
     pub(in crate) preview_messages: bool,
     pub(in crate) handle_os_events: bool,
     pub(in crate) layout_events_sender: Option<Sender<LayoutEvent>>,
@@ -366,10 +392,10 @@ pub struct Widget {
     pub(in crate) prev_arrange: Cell<Rect<f32>>,
     /// Desired size of the node after Measure pass.
     pub(in crate) desired_size: Cell<Vector2<f32>>,
-    /// Actual node local position after Arrange pass.
+    /// Actual local position of the widget after Arrange pass.
     pub(in crate) actual_local_position: Cell<Vector2<f32>>,
-    /// Actual size of the node after Arrange pass.
-    pub(in crate) actual_size: Cell<Vector2<f32>>,
+    /// Actual local size of the widget after Arrange pass.
+    pub(in crate) actual_local_size: Cell<Vector2<f32>>,
     pub(in crate) prev_global_visibility: bool,
     pub(in crate) clip_bounds: Cell<Rect<f32>>,
 }
@@ -393,7 +419,7 @@ impl Widget {
 
     #[inline]
     pub fn actual_size(&self) -> Vector2<f32> {
-        self.actual_size.get()
+        self.actual_local_size.get()
     }
 
     #[inline]
@@ -439,6 +465,15 @@ impl Widget {
         self.allow_drop
     }
 
+    #[inline]
+    pub fn screen_to_local(&self, point: Vector2<f32>) -> Vector2<f32> {
+        self.visual_transform
+            .try_inverse()
+            .unwrap_or_default()
+            .transform_point(&Point2::from(point))
+            .coords
+    }
+
     #[inline]
     pub fn invalidate_layout(&self) {
         self.invalidate_measure();
@@ -556,7 +591,7 @@ impl Widget {
 
     #[inline]
     pub fn screen_position(&self) -> Vector2<f32> {
-        self.screen_position
+        Vector2::new(self.visual_transform[6], self.visual_transform[7])
     }
 
     #[inline]
@@ -626,14 +661,34 @@ impl Widget {
 
     #[inline]
     pub fn screen_bounds(&self) -> Rect<f32> {
+        self.bounding_rect().transform(&self.visual_transform)
+    }
+
+    #[inline]
+    pub fn bounding_rect(&self) -> Rect<f32> {
         Rect::new(
-            self.screen_position.x,
-            self.screen_position.y,
-            self.actual_size.get().x,
-            self.actual_size.get().y,
+            0.0,
+            0.0,
+            self.actual_local_size.get().x,
+            self.actual_local_size.get().y,
         )
     }
 
+    #[inline]
+    pub fn visual_transform(&self) -> &Matrix3<f32> {
+        &self.visual_transform
+    }
+
+    #[inline]
+    pub fn render_transform(&self) -> &Matrix3<f32> {
+        &self.render_transform
+    }
+
+    #[inline]
+    pub fn layout_transform(&self) -> &Matrix3<f32> {
+        &self.layout_transform
+    }
+
     pub fn has_descendant(&self, node_handle: Handle<UiNode>, ui: &UserInterface) -> bool {
         for child_handle in self.children.iter() {
             if *child_handle == node_handle {
@@ -660,11 +715,14 @@ impl Widget {
     ) -> Handle<UiNode> {
         let mut parent_handle = self.parent;
         while parent_handle.is_some() {
-            let parent_node = ui.nodes.borrow(parent_handle);
-            if func(parent_node) {
-                return parent_handle;
+            if let Some(parent_node) = ui.nodes.try_borrow(parent_handle) {
+                if func(parent_node) {
+                    return parent_handle;
+                }
+                parent_handle = parent_node.parent;
+            } else {
+                break;
             }
-            parent_handle = parent_node.parent;
         }
         Handle::NONE
     }
@@ -749,6 +807,15 @@ impl Widget {
                     &WidgetMessage::Cursor(icon) => {
                         self.cursor = icon;
                     }
+                    WidgetMessage::LayoutTransform(transform) => {
+                        if &self.layout_transform != transform {
+                            self.layout_transform = *transform;
+                            self.invalidate_layout();
+                        }
+                    }
+                    WidgetMessage::RenderTransform(transform) => {
+                        self.render_transform = *transform;
+                    }
                     _ => (),
                 }
             }
@@ -828,7 +895,7 @@ impl Widget {
 
     #[inline]
     pub(in crate) fn commit_arrange(&self, position: Vector2<f32>, size: Vector2<f32>) {
-        self.actual_size.set(size);
+        self.actual_local_size.set(size);
         self.actual_local_position.set(position);
         self.arrange_valid.set(true);
     }
@@ -860,6 +927,11 @@ impl Widget {
         self.actual_local_position.get()
     }
 
+    #[inline]
+    pub fn center(&self) -> Vector2<f32> {
+        self.actual_local_position() + self.actual_size().scale(0.5)
+    }
+
     #[inline]
     pub(in crate) fn set_global_visibility(&mut self, value: bool) {
         self.prev_global_visibility = self.global_visibility;
@@ -1014,6 +1086,9 @@ pub struct WidgetBuilder {
     pub context_menu: Handle<UiNode>,
     pub preview_messages: bool,
     pub handle_os_events: bool,
+    pub layout_transform: Matrix3<f32>,
+    pub render_transform: Matrix3<f32>,
+    pub clip_to_bounds: bool,
 }
 
 impl Default for WidgetBuilder {
@@ -1054,6 +1129,9 @@ impl WidgetBuilder {
             context_menu: Handle::default(),
             preview_messages: false,
             handle_os_events: false,
+            layout_transform: Matrix3::identity(),
+            render_transform: Matrix3::identity(),
+            clip_to_bounds: true,
         }
     }
 
@@ -1077,6 +1155,11 @@ impl WidgetBuilder {
         self
     }
 
+    pub fn with_clip_to_bounds(mut self, clip_to_bounds: bool) -> Self {
+        self.clip_to_bounds = clip_to_bounds;
+        self
+    }
+
     pub fn with_enabled(mut self, enabled: bool) -> Self {
         self.enabled = enabled;
         self
@@ -1132,6 +1215,16 @@ impl WidgetBuilder {
         self
     }
 
+    pub fn with_layout_transform(mut self, layout_transform: Matrix3<f32>) -> Self {
+        self.layout_transform = layout_transform;
+        self
+    }
+
+    pub fn with_render_transform(mut self, render_transform: Matrix3<f32>) -> Self {
+        self.render_transform = render_transform;
+        self
+    }
+
     pub fn with_z_index(mut self, z_index: usize) -> Self {
         self.z_index = z_index;
         self
@@ -1231,10 +1324,9 @@ impl WidgetBuilder {
             desired_local_position: self.desired_position,
             width: self.width,
             height: self.height,
-            screen_position: Vector2::default(),
             desired_size: Cell::new(Vector2::default()),
             actual_local_position: Cell::new(Vector2::default()),
-            actual_size: Cell::new(Vector2::default()),
+            actual_local_size: Cell::new(Vector2::default()),
             min_size: self.min_size.unwrap_or_default(),
             max_size: self
                 .max_size
@@ -1273,6 +1365,10 @@ impl WidgetBuilder {
             preview_messages: self.preview_messages,
             handle_os_events: self.handle_os_events,
             layout_events_sender: None,
+            layout_transform: self.layout_transform,
+            render_transform: self.render_transform,
+            visual_transform: Matrix3::identity(),
+            clip_to_bounds: self.clip_to_bounds,
         }
     }
 }
