@@ -214,7 +214,7 @@ impl Control for Window {
 
                         // Check grips.
                         for grip in self.grips.borrow_mut().iter_mut() {
-                            let offset = self.screen_position;
+                            let offset = self.screen_position();
                             let screen_bounds = grip.bounds.translate(offset);
                             if screen_bounds.contains(pos) {
                                 grip.is_dragging = true;
@@ -239,7 +239,7 @@ impl Control for Window {
                         let mut new_cursor = None;
 
                         for grip in self.grips.borrow().iter() {
-                            let offset = self.screen_position;
+                            let offset = self.screen_position();
                             let screen_bounds = grip.bounds.translate(offset);
                             if screen_bounds.contains(pos) {
                                 new_cursor = Some(grip.cursor);
@@ -335,7 +335,7 @@ impl Control for Window {
             }
             if let WidgetMessage::Unlink = msg {
                 if message.destination() == self.handle() {
-                    self.initial_position = self.screen_position;
+                    self.initial_position = self.screen_position();
                 }
             }
         } else if let Some(ButtonMessage::Click) = message.data::<ButtonMessage>() {
@@ -451,7 +451,7 @@ impl Control for Window {
                         }
                     }
                     &WindowMessage::Move(new_pos) => {
-                        if self.desired_local_position() != new_pos {
+                        if self.is_dragging && self.desired_local_position() != new_pos {
                             ui.send_message(WidgetMessage::desired_position(
                                 self.handle(),
                                 MessageDirection::ToWidget,
@@ -551,6 +551,10 @@ impl Window {
     pub fn can_resize(&self) -> bool {
         self.can_resize
     }
+
+    pub fn content(&self) -> Handle<UiNode> {
+        self.content
+    }
 }
 
 pub struct WindowBuilder {
@@ -593,10 +597,12 @@ impl WindowTitle {
 fn make_text_title(ctx: &mut BuildContext, text: &str) -> Handle<UiNode> {
     TextBuilder::new(
         WidgetBuilder::new()
-            .with_margin(Thickness::uniform(5.0))
+            .with_margin(Thickness::left(5.0))
             .on_row(0)
             .on_column(0),
     )
+    .with_vertical_text_alignment(VerticalAlignment::Center)
+    .with_horizontal_text_alignment(HorizontalAlignment::Left)
     .with_text(text)
     .build(ctx)
 }
@@ -732,7 +738,7 @@ impl<'a> WindowBuilder {
         let header = BorderBuilder::new(
             WidgetBuilder::new()
                 .with_horizontal_alignment(HorizontalAlignment::Stretch)
-                .with_height(30.0)
+                .with_height(25.0)
                 .with_background(Brush::LinearGradient {
                     from: Vector2::new(0.5, 0.0),
                     to: Vector2::new(0.5, 1.0),
